/*
** Module   : SessionUtils.java
** Abstract : Defines APIs used to access the SESSION attributes and methods.
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --------------------------------Description-----------------------------------
** 001 CA  20130112 Created initial version.
** 002 CS  20130214 Added support for SESSION: EXPORT, SERVER-OPERATING-MODE, REMOTE attributes.
** 003 CA  20130221 getType was renamed to getResourceType.
** 004 CA  20130222 Added stubs for more SESSION-related attributes and methods: REMOTE,
**                  PARAMETER. The implementations for EXPORT, SERVER-OPERATING-MODE and REMOVE 
**                  attrs call their ServerImpl counterparts.
** 005 OM  20130304 Refactored isValid and isUnknown of WrappedResource to valid and unknown.
** 006 OM  20130510 Added support for date-related SESSION attributes so they can be queried from 
**                  client side.
*/

package com.goldencode.p2j.util;

import com.goldencode.p2j.net.*;
import com.goldencode.p2j.security.ContextLocal;
import com.goldencode.p2j.ui.LogicalTerminal;
import com.goldencode.proxy.StaticProxy;

/**
 * This class provides implementation for SESSION attributes and methods.
 * Mainly, the APIs implemented here are defined in the {@link CommonSession}
 * interface (or one of its super interfaces) and are not statically 
 * implemented by the classes used by the {@link #asHandle()}.
 */
public class SessionUtils
{
   /**
    * Get a the instance for the SESSION system handle.  Is obtained using
    * a call to the {@link StaticProxy#obtain(Class, Class[])}, using the
    * {@link CommonSession} interface and its methods implemented by these
    * classes: {@link SessionUtils}, {@link EnvironmentOps}, {@link date},
    * {@link LogicalTerminal}, {@link ProcedureManager}.
    * 
    * @return   See above.
    */
   public static handle asHandle()
   {
      Class[] classes = 
      {
         SessionUtils.class,
         EnvironmentOps.class,
         date.class,
         LogicalTerminal.class,
         ProcedureManager.class,
         NumberType.class
      };
      CommonSession proxy = StaticProxy.obtain(CommonSession.class, classes);
      return new handle(proxy);
   }
   
   /**
    * Get the type of its associated handle.
    * 
    * @return   Always return the PSEUDO-WIDGET value.
    */
   public static character getResourceType()
   {
      return new character("PSEUDO-WIDGET");
   }
   
   /**
    * Implementation for the {@link WrappedResource#valid()} API.
    * 
    * @return   Always true.
    */
   public static boolean valid()
   {
      return true;
   }
   
   /**
    * Implementation for the {@link WrappedResource#unknown()} API.
    * 
    * @return   Always false.
    */
   public static boolean unknown()
   {
      return false;
   }
   
   /**
    * API needed to implement read-only attribute assignment (a 4GL 
    * "feature").
    * 
    * @param    attribute
    *           The attribute's name.
    *           
    * @see      handle#readOnlyError(handle, String)
    */
   public static void readOnlyError(String attribute)
   {
      handle.readOnlyError(asHandle(), attribute);
   }
   
   /**
    * Indicates if the session is for the context of an appserver.
    * 
    * @return   <code>true</code> if the session is for the context of an appserver.
    */
   public  static logical isRemote()
   {
      return ServerImpl.isRemote();
   }
   
   /**
    * Method implementation for getting the SERVER-OPERATING-MODE attribute of the session handle.
    * <p>
    * Possible values:
    * <ul>
    *    <li>State-reset
    *    <li>State-aware
    *    <li>Stateless
    *    <li>State-free
    * </ul>
    * 
    * @return   The corresponding SERVER-OPERATING-MODE attribute of the session handle.
    */
   public static character getServerOperatingMode()
   {
      return ServerImpl.getServerOperatingMode();
   }
   
   /**
    * Method that specifies the remote procedures the client can execute from the AppServer 
    * session.
    * 
    * @param    attrList
    *           The list of procedure  and  name-patterns separated by comma to be exported.
    *         
    * @return   <true> if REMOTE attribute is true and operation succeeded or
    *           <false> otherwise
    */
   public static logical export(String attrList)
   {
      return ServerImpl.export(attrList);
   }
   
   /**
    * Method that specifies the remote procedures the client can execute from the AppServer 
    * session.
    * 
    * @param    attrList
    *           The list of procedure  and  name-patterns separated by comma to be exported.
    *         
    * @return   <code>true</code> if REMOTE attribute is true and operation succeeded or
    *           <code>false</code> otherwise
    */
   public static logical export(character attrList)
   {
      return ServerImpl.export(attrList);
   }

   /**
    * Get the value of the PARAMETER attribute, which is set to the value of the Parameter 
    * (-param) startup parameter specified for the current session.
    * 
    * @return   See above.
    */
   public static character getParameter()
   {
      // TODO: implement this
      return null;
   }

   /**
    * Returns the handle for the first dynamic buffer in the first table containing a dynamic 
    * buffer.
    * <p>
    * TODO: this should call a worker method from the persist package.
    * 
    * @return   See above.
    */
   public static handle firstBuffer()
   {
      // TODO: implement this
      return null;
   }
   
   /** 
    * Sets the order of date components (SESSION:DATE-FORMAT) for conversion to/from strings.
    * <p>
    * The order of the 3 date sub-components as a 3 character string using "M", "D" and "Y" once 
    * each.  The order from index 0 to index 2 represents the left to right ordering of the 
    * components. So the leftmost date component will be defined by the character at index 
    * position 0.
    * <p>
    * The value set is specific to the current user's context.
    *
    * @param    dateFormat
    *           Defines the order of date components for conversion to/from strings.
    */
   public static void setDateOrder(character dateFormat)
   {
      date.setDateOrder(dateFormat);
      exports.get().setDateOrder(dateFormat);
   }
   
   /** 
    * Gets the order of date components (SESSION:DATE-FORMAT) for conversion to/from strings.
    * <p>
    * The order of the 3 date sub-components is a 3 character string using
    * "M", "D" and "Y" once each.  The order from index 0 to index 2
    * represents the left to right ordering of the components. So the leftmost
    * date component will be defined by the character at index position 0.
    * <p>
    * The value accessed is specific to the current user's context.
    *
    * @return   Defines the order of date components for conversion to/from
    *           strings.
    */
   public static character getDateOrder()
   {
      return date.getDateOrder();
   }

   /**
    * Sets the SESSION:TIMEZONE used as the default offset in minutes from
    * UTC for this user's context.
    *
    * @param    offset
    *           The new timezone offset which must be between -840 and 840
    *           inclusive.  The value may also be <code>unknown</code> which
    *           is the equivalent of resetting this attribute to the default.
    */
   public static void setSessionTimeZone(NumberType offset)
   {
      date.setSessionTimeZone(offset);
      exports.get().setSessionTimeZone(offset);
   }
   
   /**
    * Accesses the SESSION:TIMEZONE used as the default offset in minutes from
    * UTC for this user's context.
    *
    * @return   The timezone offset. If this has never been set, then
    *           <code>unknown</code> will be returned.
    */
   public static integer getSessionTimeZone()
   {
      return date.getSessionTimeZone();
   }

   /**
    * Obtains the currently configured TIME-SOURCE attribute for the current
    * user's context.  This attribute is can be read and written but it is
    * not honored for date/time processing in P2J.  The intention of the
    * attribute is to define the system from which time values will be obtained.
    * In P2J, time values are always obtained from the application server. 
    * <p>
    * The default value is "local" which in Progress means that the client
    * machine is the source  (this is the same as the empty string "").  It is
    * common to use a database name in the 4GL, which makes the application
    * use the database server as the time source.
    *
    * @return   The time source attribute.
    */
   public static character getTimeSource()
   {
      return date.getTimeSource();
   }
   
   /**
    * Sets the <code>TIME-SOURCE</code> attribute for the user's context.
    * This attribute is can be read and written but it is not honored for 
    * date/time processing in P2J.  The intention of the attribute is to define
    * the system from which time values will be obtained. In P2J, time values
    * are always obtained from the application server.
    *
    * @param    timeSource
    *           The new value to set. If <code>null</code>, then the empty
    *           string will be used, which in the 4GL has the same affect as
    *           "local".
    */
   public static void setTimeSource(character timeSource)
   {
      date.setTimeSource(timeSource);
      exports.get().setTimeSource(timeSource);
   }
   
   /**
    * Obtains the currently configured DISPLAY-TIMEZONE attribute for the current
    * user's context. This attribute is used for displaying DATETIME-TZ values that do not have 
    * the timezone specified in the display format. 
    *
    * @return   The display-timezone attribute.
    */
   public static integer getDisplayTimeZone()
   {
      return date.getDisplayTimeZone();
   }
   
   /**
    * Sets the <code>DISPLAY-TIMEZONE</code> attribute for the user's context.
    * This attribute is used for displaying DATETIME-TZ values that do not have 
    * the timezone specified in the display format. 
    *
    * @param    displayTimezone
    *           The new value to set.
    */
   public static void setDisplayTimeZone(int64 displayTimezone)
   {
      date.setDisplayTimeZone(displayTimezone);
      exports.get().setDisplayTimeZone(displayTimezone);
   }
   
   /**
    * Implementation of SESSION:YEAR-OFFSET attribute getter. Returns the 
    * currently configured start year for the two-digit Y2K windowing range.
    * This is the equivalent to the Progress -yy command line parameter. The 
    * default value is 1950 unless otherwise specified in the directory (see
    * {@link date.getWindowingYear}).
    * 
    * @return  The start year for the two-digit windowing range.
    */
   public static integer getYearOffset()
   {
      return date.getYearOffset();
   }
   
   /**
    * Implementation of SESSION:YEAR-OFFSET attribute setter. Sets the 
    * configured start year for the two-digit Y2K windowing range. This is
    * the equivalent of the Progress -yy command line parameter. The 
    * default value is 1950 unless otherwise specified in the directory (see
    * {@link date.getWindowingYear}).
    * 
    * @param   year
    *          The new start year for the two-digit windowing range.
    */
   public static void setYearOffset(NumberType year)
   {
      date.setYearOffset(year);
      exports.get().setYearOffset(year);
   }
   
   
   private static final ContextLocal<SessionExports> exports =
         new ContextLocal<SessionExports>()
         {
            @Override
            protected SessionExports initialValue()
            {
               Session session = SessionManager.get().getSession();
               return (SessionExports) ((session != null) ?
                     RemoteObject.obtainNetworkInstance(SessionExports.class) :
                     RemoteObject.obtainLocalInstance(SessionExports.class, true));
            }
         };
}